 { simple program to test Chebyshev approximation
   of the precession-nutation parameters  psi epsilon X Y s(t) }
 Program ToTestApproxNutationParameters ;
 Uses
   UNConTyp , { types, constants, simple procedure }
   UN2000_L , { nutation luni-solar terms }
   UN2000_P , { nutation planetary  terms }
   UNXY2000 , { the value of  X , Y  parameters }
   Un2000_s ; { for function ToGetCEOPos for s(t) in ArcSec }
 Const
   InterPolM  = 11 ;
   InterPoint = InterPolM+2 ;
   NumOfParm  = 5 ;
 Type
   TOneApp    = Array[1..InterPolM+1] of Double ;
   TApprox    = Record
                  BegApp  : Double ;
                  EndApp  : Double ;
                  PosApp  : Array [1..NumOfParm] of TOneApp ;
                End;
 Var
   polc    : Array [1..InterPolM+1] of Extended ;
   xuzpo   : Array [1..InterPoint] Of Extended ;
   polcpo  : Array [1..InterPolM+1,1..InterPoint] of Extended ;
 Var
   AppFile      : File of TApprox ;
   ApproxC      : TApprox ;
   TApproxInit  : Extended ;
   dpsi,deps    : Extended ; { full nutation }
   dpsil,depsl  : Extended ; { luni-solar nutation }
   dpsip,depsp  : Extended ; { planetary nutation }
   xp,yp,ceops  : Extended ;
   XParm,YParm  : Extended ;
   ParmCur      : Array [1..NumOfParm] of Extended ;
   VectorXYZ    : TVect3 ;   { array[1..3] , type in Un2000_C }
   date1,date2  : Extended ;
   StepApp      : Extended ;
   TBeg,TEnd    : Extended ;
   NApproxRec   : LongInt ;
   NPosError    : LongInt ;

 { to calculate Chebyshev polinoms }

 Procedure ClcChebPol ( n  : Byte ; x  : Extended ) ;
 Var
   i    : Byte ;
 Begin
   polc[1]:=1;
   polc[2]:=x;
   For i:=2 To n Do
     polc[i+1]:=2*x*polc[i]-polc[i-1];
 End;

 { to calculate the values of the Chebyshev polinoms
      in the selected points }

 Procedure ClcChebPolinPoints ;
 Var
   i,j,k,n            : Byte;
   prnpo,xuz          : Extended;
 Begin
   n:=InterpolM;
   prnpo:=0.5e0*2*Pi/(n+1.0e0);
   For k:=1 To InterpolM+1 Do
     Begin
       polc[k]:=0;
       For i:=1 To InterPoint Do
         Begin
           If  k = 1  Then  xuzpo[i]:=0;
           polcpo[k,i]:=0;
         End;
     End;
   For i:=1 To InterPoint Do
     Begin
       j:=InterPoint-i;
       xuz:=Cos(j*prnpo);
       ClcChebPol(n,xuz);
       For k:=1 To InterpolM+1 Do  polcpo[k,i]:=polc[k];
       xuzpo[i]:=xuz;
     End;
 End;

 { to use Chebyshev coefficients of the approximation }

 Procedure ChebApprox ( m  : Byte ;  t  : Extended ) ;
 Var
   i,j        : Byte;
   sx,tm,x    : Extended;
 Begin
   sx:=0.5*(ApproxC.BegApp+ApproxC.EndApp);
   tm:=0.5*(ApproxC.EndApp-ApproxC.BegApp);
   x:=(t-sx)/tm; ClcChebPol(m,x);
   For j:=1 To NumOfParm Do
     Begin
       ParmCur[j]:=0;
       For i:=1 To m+1 Do
         ParmCur[j]:=ParmCur[j]+ApproxC.PosApp[j][i]*polc[i];
     End;
 End;

 { to read file  ApproxFile
   with the values of the coefficients
   that appeared from the Chebyshev approximarion process
   and to obtain the values of the parameters on the moment  TCur }

 Procedure ToReadApproxFile ( TCur       : Extended ;
                              Var  NPos  : LongInt  ) ;
 Var
   dt,r  : Extended ;
 Begin
   If  ( ( TCur < ApproxC.BegApp ) OR ( TCur > ApproxC.EndApp ) )
     Then
       Begin
         r:=(TCur-TApproxInit)/StepApp;
         NPos:=Trunc(r);
         If  NPos < NApproxRec
           Then
             Begin
               Seek(AppFile,NPos);
               Read(AppFile,ApproxC);
             End
           Else
             Begin
               NPos:=-1; { error }
               Exit;
             End;
       End;
   ChebApprox(InterpolM,TCur); { result in ParmCur }
 End;

 Begin { main approximation }
   ClcChebPolinPoints ; { to prepare some arrays }
   Assign(AppFile,'parmaps.pat');
   ReSet(AppFile);
   Read(AppFile,ApproxC);
   TApproxInit:=ApproxC.BegApp;
   StepApp:=ApproxC.EndApp-ApproxC.BegApp; {  interval in day }
   date1:=ApproxC.BegApp;
   NApproxRec:=FileSize(AppFile);
   Seek(AppFile,NApproxRec-1);
   Read(AppFile,ApproxC);
   date2:=ApproxC.BegApp;
   TBeg:=date1+0.17;
   ReSet(AppFile);
   Read(AppFile,ApproxC);
   While  TBeg < date2  Do
     Begin
       ToReadApproxFile(TBeg,NPosError); { result in ParmCur }
       NutParmLuSo(TBeg,dpsil,depsl); { from Un2000_L in ArcSec }
       NutParmPlan(TBeg,dpsip,depsp); { from Un2000_P in ArcSec }
       dpsi:=dpsil+dpsip; { nutation in longitude ArcSec }
       deps:=depsl+depsp; { nutation in obliquity ArcSec }
       ForPNPoleXY(TBeg,VectorXYZ); { unit vector of the pole }
       xp:=VectorXYZ[1]; { in radian }
       yp:=VectorXYZ[2]; { in radian }
       ceops:=ToGetCEOPos(TBeg,xp,yp); { s(t) in ArcSec }
       XParm:=RadSec*VectorXYZ[1]; { X from radian to ArcSec }
       YParm:=RadSec*VectorXYZ[2]; { Y from radian to ArcSec }
       WriteLn((TBeg-JD2000):9:3,
               (dpsi-ParmCur[1]):13:8,
               (deps-ParmCur[2]):12:8,
               (XParm-ParmCur[3]):13:8,
               (YParm-ParmCur[4]):12:8,
               (ceops-ParmCur[5]):13:8);
       TBeg:=TBeg+0.13;
     End;
   Close(AppFile);
 End.