
 { for the simple mathematical functions }

 UNIT UnForFun ;

 INTERFACE

 Uses
   UnConTyp ;

 Type
   TCompValue  = Record
                   rval  : Extended ;
                   ival  : Extended ;
                 End ;

 Function DATan2 ( SinAngle , CosAngle  : Extended ) : Extended ;

 Function ATanDegree ( s , c  : Extended ) : Extended ;

 Function  Log10 ( x   : Extended ) : Extended ;

 Procedure MultComplex ( x , y  : TCompValue ;
                         Var z  : TCompValue ) ;

 Function ScalarMult ( a , b  : TVect3 ) : Extended ;

 Procedure ForRotMatr ( iaxis  : Byte ;
                        angle  : Extended ;
                        Var  matr  : TMatr33 ) ;

 Procedure ToMultMatr ( ma  : TMatr33 ;
                        mb  : TMatr33 ;
                        Var  mc  : TMatr33 ) ;

 Procedure MultMatrVec ( p  : TMatr33 ;
                         v  : TVect3 ;
                         Var  vp  : TVect3 ) ;

 IMPLEMENTATION

 { DATan2 is a function
   to obtain Arctan for the Angle in interval
      from  -(1/2)*Pi  to  +(3/2)*Pi  in rad }

 Function DATan2 ( SinAngle , CosAngle  : Extended ) : Extended ;
 Var
   a,c,s,p : Extended;
 Begin
   c:=CosAngle; s:=SinAngle;
   if  c <> 0 then  Begin
      p:=s/c;  a:=ArcTan(p);
      if  c < 0  then  a:=a+Pi;  End
              else  if  s > 0  then  a:=Pi/2  else  a:=-Pi/2;
   DATan2:=a;
 End;

 { ATanDegree is a function  to obtain
   Arctan for the Angle in interval from  0  to  360  in degree }

 Function ATanDegree ( s , c  : Extended ) : Extended ;
 Var
   a  : Extended;
 Begin
   a:=(180/Pi)*DATan2(s,c); { in degree }
   If  a < 0  Then  a:=360+a;
   ATanDegree:=a;
 End;

 { from the interesting book }

 Function  Log10 ( x   : Extended ) : Extended ;
 Begin
   Log10:=Ln(x)/Ln(10);
 End;

 Procedure MultComplex ( x , y  : TCompValue ;
                         Var z  : TCompValue ) ;
 Begin
   z.rval:=x.rval*y.rval-x.ival*y.ival;
   z.ival:=x.rval*y.ival+x.ival*y.rval;
 End;

 Function ScalarMult ( a , b  : TVect3 ) : Extended ;
 Begin
   ScalarMult:=a[1]*b[1]+a[2]*b[2]+a[3]*b[3];
 End;

 { procedure to fill the elements of the simple rotation matrix }

 Procedure ForRotMatr ( iaxis  : Byte ;
                        angle  : Extended ;
                        Var  matr  : TMatr33 ) ;
 Var
   i,j  : Byte ;
   s,c  : Extended ;
 Begin
   c:=Cos(angle);
   s:=Sin(angle);
   For i:=1 To 3 Do
     For j:=1 To 3 Do
       If  i = j
         Then
           matr[i,j]:=1.0
         Else
           matr[i,j]:=0.0; { unit matrix }
   Case  iaxis  Of
     1 : Begin { rotate around OX axis }
           matr[2,2]:=+c;
           matr[2,3]:=+s;
           matr[3,2]:=-s;
           matr[3,3]:=+c;
         End;
     2 : Begin { rotate around OY axis }
           matr[1,1]:=+c;
           matr[1,3]:=-s;
           matr[3,1]:=+s;
           matr[3,3]:=+c;
         End;
     3 : Begin { rotate around OZ axis }
           matr[1,1]:=+c;
           matr[1,2]:=+s;
           matr[2,1]:=-s;
           matr[2,2]:=+c;
         End;
   End; { case for axis of rotation }
 End;

 { procedure to multiple to simple 3*3 matrix }

 Procedure ToMultMatr ( ma  : TMatr33 ;
                        mb  : TMatr33 ;
                        Var  mc  : TMatr33 ) ;
 Var
   i,j,k  : Byte ;
 Begin
   For i:=1 To 3 Do
     For j:=1 To 3 Do
       Begin
         mc[i,j]:=0.0;
         For k:=1 To 3 Do
           mc[i,j]:=mc[i,j]+ma[i,k]*mb[k,j];
       End;
 End;

 Procedure MultMatrVec ( p  : TMatr33 ;
                         v  : TVect3 ;
                         Var  vp  : TVect3 ) ;
 Var
   i,j  : Byte ;
 Begin
   For i:=1 To 3 Do
     Begin
       vp[i]:=0.0;
       For j:=1 To 3 Do
         vp[i]:=vp[i]+p[i,j]*v[j];
     End;
 End;

 END.