
 { in units
   UnConTyp  are types and constants
   UnForFum  are simple functions for vectors and matrixes
   Un2000_C  are simple procedures for arguments
   Un2000_L  is procedure to calculate the Luni-Solar nutation
   Un2000_P  is procedure to calculate the planetary terms
   UnPN2000  are the procedures to obtain matrixes.
   a try to calculate with MHB2000A precession/nutation theory
   Resolution B1.6
   of General Assembly of the International Astronomical Union 
   from  IERS Conventions 2000 , web site
   http:\\maia.usno.navy.mil\conv2000.html }

 UNIT UnPN2000 ;

 INTERFACE

 Uses
   UnConTyp ; { for TMatr33 }

 Procedure PrecMatr2000 ( DateJulian  : Extended ;
                          Var  PrecM  : TMatr33  ) ;

 Procedure NutMatr2000  ( DateJulian  : Extended ;
                          Var   NutM  : TMatr33  ) ;

 Procedure ForPNMatrix ( DateJulian   : Extended ;
                         Var  PNMatr  : TMatr33 ;
                         Var  PNPrim  : TMatr33 ) ;

 IMPLEMENTATION

 Uses
   UnForFun , { for simple functions ForRotMatr , ToMultMatr }
   Un2000_L , Un2000_P ; { nutation }
 
 { PrecM is matrix of precession P to change
   from the standard equator to the moving equator
   P= R3(xiA) * R1(-omegaA) * R3(-psiA) * R1(epsilon0) }

 Procedure PrecMatr2000 ( DateJulian  : Extended ;
                          Var  PrecM  : TMatr33  ) ;
 Var
   t,epso,psia,oma,xia  : Extended ;
   matr1,matr2,matr3,matr4,matra,matrb  : TMatr33 ;
 Begin
   t:=(DateJulian-JD2000)/JulianC; { from the fundamental epoch in century }
   epso:=SecRad*84381.448; { in radian }
   psia:=SecRad*(5038.47875+(-1.07259-0.001147*t)*t)*t;
   oma:=epso+SecRad*(-0.02542+(0.05127-0.007726*t)*t)*t;
   xia:=SecRad*(10.5526+(-2.38064-0.001125*t)*t)*t;
   ForRotMatr(1,epso,matr1);
   ForRotMatr(3,-psia,matr2);
   ForRotMatr(1,-oma,matr3);
   ForRotMatr(3,xia,matr4);
   ToMultMatr(matr2,matr1,matra);
   ToMultMatr(matr3,matra,matrb);
   ToMultMatr(matr4,matrb,PrecM);
 End;

 { NutM is matrix of nutation N to change
   from the moving equator to true equator
   N = R1(-epsilon-depsilon) * R3(-dpsi) * R1(epsilon) }

 Procedure NutMatr2000  ( DateJulian  : Extended ;
                          Var   NutM  : TMatr33  ) ;
 Var
   t,epsa     : Extended ;
   dplu,delu  : Extended ;
   dppl,depl  : Extended ;
   dpsi,deps  : Extended ;
   epso,epst  : Extended ;
   mata,matb  : TMatr33 ;
   matc,matd  : TMatr33 ;
 Begin
   t:=(DateJulian-JD2000)/JulianC; { from the fundamental epoch in century }
   NutParmLuSo(DateJulian,dplu,delu);
   NutParmPlan(DateJulian,dppl,depl);
   dpsi:=SecRad*(dplu+dppl); { from arcsec to radian }
   deps:=SecRad*(delu+depl);
   epso:=SecRad*84381.448; { in arcsec }
   epsa:=epso+SecRad*(-46.84024+(-0.00059+0.001813*t)*t)*t;
   epst:=epsa+deps;
   ForRotMatr(1,epsa,mata) ;
   ForRotMatr(3,-dpsi,matb);
   ForRotMatr(1,-epst,matc);
   ToMultMatr(matb,mata,matd);
   ToMultMatr(matc,matd,NutM);
 End;

 Procedure ToPrint ( s  : String ;  p : TMatr33 ) ;
 Var
   i,j  : Byte ;
 Begin
   Write(s:10);
   For i:=1 To 3 Do  Write(i:15);
   WriteLn;
   For i:=1 To 3 Do
     Begin
       Write(i:10);
       For j:=1 To 3 Do
         Write(p[i,j]:15:5);
       WriteLn;
     End;
 End;

 { PNMatr is matrix to change from the standard to moving equator
   PNPrim is matrix to change from the moving to standard equator }

 Procedure ForPNMatrix ( DateJulian   : Extended ;
                         Var  PNMatr  : TMatr33 ;
                         Var  PNPrim  : TMatr33 ) ;
 Var
   i,j  : Byte ;
   precm,nutm  : TMatr33 ;
 Begin
   PrecMatr2000(DateJulian,precm);
   NutMatr2000(DateJulian,nutm);
   ToMultMatr(nutm,precm,PNMatr);
   For i:=1 To 3 Do
     For j:=1 To 3 Do
       PNPrim[i,j]:=PNMatr[j,i];
 End;

 END.
