
 { in unit  UnZFiles
   are the procedures to obtain
       the values for the parameters X , Y
       the values for s(t) and Greenwich Sidereal Time GMT
       the values for nutation parameters dPsi and dEpsilon
       coefficients are read from the files
       N2000_X.TXT , N2000_Y.TXT , N2000_S.TXT , 
       N2000_3.TXT , N2000_1.TXT , N2000_2.TXT }

 UNIT UnZFiles ;

 INTERFACE

 { to obtain X and Y just reading coefficients from file
   uncorrect file with coefficients especially for Y }

 Procedure ForValueXY  ( DateJulian  : Extended ;
                         Var   XCip  : Extended ;
                         Var   YCip  : Extended ) ;

 { to obtain value for s(t) by using the coefficients from the file }

 Function ValueCEOPos ( DateJulian  : Extended ) : Extended ;

 { to obtain value for GST by using the coefficients from the file }

 Function ToGetGSideRealTime ( DateJulian  : Extended ) : Extended ;

 { Luni-Solar nutation parameter , coefficients on disk }

 Procedure NutReadLuSo  ( DateJulian  : Extended ;
                          Var   DPsi  : Extended ;
                          Var   DEps  : Extended ) ;

 { Planetary nutation parameter , coefficients on disk }

 Procedure NutReadPlan  ( DateJulian  : Extended ;
                          Var   DPsi  : Extended ;
                          Var   DEps  : Extended ) ;

 IMPLEMENTATION

 Uses
   UnConTyp , Un2000_r ,
   Un2000_C , Un2000_L , Un2000_P , UnXY2000 ;

 Var
   fr  : Text ;

 Function NTermsCurrent ( j  : Byte )  : Word ;
 Var
   nc  : Word ;
   np  : Byte ;
   nz  : Byte ;
   ne  : Integer ;
   sj  : String [1] ;
   sc  : String [5] ;
   sp  : String [29] ;
 Begin
   If  EOF(fr)
     Then
       Begin
         NTermsCurrent:=0;
         Exit ; { no action }
       End;
   nc:=0;
   np:=0;
   Str(j:1,sj); { value of j (0..4) to string }
   sc:='j = '+sj; { string to compare for example if j=1 then sc:='j = 1' }
   Repeat
     ReadLn(fr,sp);
     np:=Pos(sc,sp); { to find substring in current string }
   Until  ( ( np = 1 ) OR EOF(fr) ) ;
   If  np = 1
     Then
       Begin
         Delete(sp,1,21); { to delete substring 'j = 0  Nb of terms = ' }
         Repeat { to delete possible blank in numerical string }
           nz:=Pos(' ',sp);
           If  nz > 0
             Then
               Delete(sp,nz,1);
         Until  nz = 0 ;
         Val(sp,nc,ne); { number of terms }
         ReadLn(fr); { one nullo string }
       End;
   NTermsCurrent:=nc;
 End;

 Function TermCurrent ( farg  : TFundArg ) : Extended ;
 Var
   i    : Byte ;
   nc   : Word ;
   na   : Array [1..14] of ShortInt ;
   arg  : Extended ;
   a,b  : Extended ;
   s,c  : Extended ;
 Begin
   arg:=0;
   Read(fr,nc,a,b);
   For i:=1 To 13 Do  Read(fr,na[i]);
   ReadLn(fr,na[14]);
   For i:=1 To 14 Do
     arg:=arg+na[i]*farg[i]; { argument in radian }
   c:=Cos(arg);
   s:=Sin(arg);
   TermCurrent:=a*s+b*c; { in microarcsec }
 End;

 Function ValueFromFile ( t  : Extended ;  farg  : TFundArg ) : Extended ;
 Var
   j  : Byte ;
   k  : Word ;
   n  : Word ;
   q  : Extended ;
   r  : Extended ;
 Begin
   r:=0.0;
   q:=1.0;
   ReSet(fr);
   For j:=0 To 4 Do
     Begin
       n:=NTermsCurrent(j);
       For k:=1 To n Do
         r:=r+TermCurrent(farg)*q;
       q:=q*t;
     End;
   Close(fr);
   ValueFromFile:=1.0e-6*r; { in arcsec }
 End;

 { to obtain value for X and Y position
   of the CIP in the GCRS by using the files
   with the coefficients  n2000_x.txt , n2000_y.txt }

 Procedure ForValueXY  ( DateJulian  : Extended ;
                         Var   XCip  : Extended ;
                         Var   YCip  : Extended ) ;
 Var
   t,d    : Extended ;
   farg   : TFundArg ;
   xp,xr  : Extended ;
   yp,yr  : Extended ;
 Begin
   d:=DateJulian;
   FundArgLuSo(d,farg);
   FundArgPlan(d,farg);
   t:=(d-JD2000)/JulianC; { from the fundamental epoch in century }
   { polynomial part for X , from microarcsec to arcsec }
   xp:=1.0e-6*(-16616.99+(+2004191742.88
                        +(-427219.05+(-198620.54+(-46.05+5.98*t)*t)*t)*t)*t);
   Assign(fr,'n2000_x.txt'); { file with nonpolynomial part of X }
   xr:=ValueFromFile(t,farg); { nonpolynomial part for X in arcsec }
   XCip:=xp+xr; { in arcsec }
   { polynomial part for Y , from microarcsec to arcsec }
   yp:=1.0e-6*(-6950.78+(-25381.99+(-22407250.99
                       +(+1842.28+(+1113.06+0.99*t)*t)*t)*t)*t);
   Assign(fr,'n2000_y.txt'); { file with nonpolynomial part of Y }
   yr:=ValueFromFile(t,farg); { nonpolynomial part for Y in arcsec }
   YCip:=yp+yr; { in arcsec }
 End;

 { to obtain function s(t) by using the series from the file }

 Function ValueCEOPos ( DateJulian  : Extended ) : Extended ;
 Var
   t,d    : Extended ;
   farg   : TFundArg ;
   sp,sr  : Extended ;
   xc,yc  : Extended ;
 Begin
   d:=DateJulian;
   FundArgLuSo(d,farg);
   FundArgPlan(d,farg);
   t:=(d-JD2000)/JulianC; { from the fundamental epoch in century }
   { polynomial part for s(t)+XY , from microarcsec to arcsec }
   sp:=1.0e-6*(94.0+(3808.35
                   +(-119.94+(-72574.09+(+27.70+15.61*t)*t)*t)*t)*t);
   Assign(fr,'n2000_s.txt'); { file with nonpolynomial part of s(t) }
   sr:=ValueFromFile(t,farg); { nonpolynomial part for s(t) in arcsec }
   FormulaeXY(d,xc,yc); { in radian }
   ValueCEOPos:=-0.5*RadSec*xc*yc+sp+sr; { in arcsec }
 End;

 { Expression for Greenwich Sidereal Time
   based on the IAU2000A precession-nutation model
   Expression ensuring continuity of UT1 on 1st January 2003
   GST = Theta(UT1) + polynomial part
                    + DeltaPsi*cos(epsilon_A) + non-polynomial part
   Theta(UT1) = 2*Pi*(0.7790572732640 + 1.00273781191135448.T_u)
   where T_u = Julian UT1 date - 2451545.0, and UT1 = UTC + (UT1 - UTC)
   Polynomial part (unit arcsecond)
   0''.014506 + 4612''.15739966t 
              + 1''.39667721t^2 - 0''.00009344t^3 + 0''.00001882t^4 }

 Function ToGetGSideRealTime ( DateJulian  : Extended ) : Extended ;
 Var
   farg   : TFundArg ;
   d,t,pp,pr,rg,rr,rq   : Extended ;
   epso,epsa,dpsi,teta  : Extended ;
   dplu,delu,dppl,depl  : Extended ;
 Begin
   d:=DateJulian;
   { teta for the Earth rotation angle in radian }
   teta:=ToGetEarthRotAngle(DateJulian);
   FundArgLuSo(d,farg);
   FundArgPlan(d,farg);
   t:=(DateJulian-JD2000)/JulianC; { in century }
   pp:=0.014506+(+4612.15739966
               +(+1.39667721+(-0.00009344+0.00001882*t)*t)*t)*t;
   pp:=SecRad*pp;  { polynomial part in radian }
   Assign(fr,'n2000_3.txt'); { file with nonpolynomial part of GST }
   pr:=SecRad*ValueFromFile(t,farg); { nonpolynomial part for GST in radian }
   epso:=SecRad*84381.448; { in radian }
   epsa:=epso+SecRad*(-46.84024+(-0.00059+0.001813*t)*t)*t; { in radian }
   NutParmLuSo(DateJulian,dplu,delu);
   NutParmPlan(DateJulian,dppl,depl);
   dpsi:=SecRad*(dplu+dppl); { from arcsec to radian }
   rr:=teta+pp+dpsi*Cos(epsa)+pr;
   rq:=Int(rr/(2*Pi));
   rg:=rr-2*Pi*rq;
   If  rg < 0  Then  rg:=rg+2*Pi;
   ToGetGSideRealTime:=rg;
 End;

 { to use the numbers from disk but with big time to read disk }

 Procedure NutReadLuSo  ( DateJulian  : Extended ;
                          Var   DPsi  : Extended ;
                          Var   DEps  : Extended ) ;
 Var
   i       : Byte ;
   t       : Extended ;
   narg    : Word ;
   argn    : TFundArg ;
   carg    : Extended ;
   sarg    : Extended ;
   numa    : Array [1..5] of ShortInt ;
   argum   : Extended ;
   apsis   : Extended ;
   dpsis   : Extended ;
   aepsc   : Extended ;
   depsc   : Extended ;
   apsic   : Extended ;
   dpsic   : Extended ;
   aepss   : Extended ;
   depss   : Extended ;
   period  : Extended ;
 Begin
   Assign(fr,'n2000_1.txt');
   ReSet(fr);
   FundArgLuSo(DateJulian,argn);
   t:=(DateJulian-JD2000)/JulianC;
   DPsi:=0.0;
   DEps:=0.0;
   For i:=1 To 4 Do  ReadLn(fr);
   For narg:=1 To 678 Do
     Begin
       ReadLn(fr,numa[1],numa[2],numa[3],numa[4],numa[5],
                    period,
                    apsis,dpsis,aepsc,depsc,
                    apsic,dpsic,aepss,depss);
       argum:=0.0;
       For i:=1 To 5 Do
         argum:=argum+numa[i]*argn[i];
       carg:=Cos(argum);
       sarg:=Sin(argum);
       DPsi:=DPsi+(apsis+t*dpsis)*sarg+(apsic+t*dpsic)*carg;
       DEps:=DEps+(aepsc+t*depsc)*carg+(aepss+t*depss)*sarg;
     End;
   DPsi:=1.0e-3*Dpsi; { in arcsec }
   DEps:=1.0e-3*DEps; { in arcsec }
   Close(fr);
 End;

 { to obtain the influence of the nutation planetary terms
   but the coefficients are reading from the disk }

 Procedure NutReadPlan  ( DateJulian  : Extended ;
                          Var   DPsi  : Extended ;
                          Var   DEps  : Extended ) ;
 Var
   i      : Byte ;
   narg   : Word ;
   argn   : TFundArg ;
   carg   : Extended ;
   sa,ca  : Extended ;
   nn     : Word ;
   pt     : Extended ;   
   numa   : Array[1..14] of ShortInt ;
   apsis  : Extended ;
   apsic  : Extended ;
   aepsc  : Extended ;
   aepss  : Extended ;
 Begin
   Assign(fr,'n2000_2.txt');
   ReSet(fr);
   For nn:=1 To 5 Do  ReadLn(fr);
   FundArgLuSo(DateJulian,argn);
   FundArgPlan(DateJulian,argn);
   DPsi:=0.0;
   DEps:=0.0;
   For narg:=1 To 687 Do
     Begin
       ReadLn(fr,nn,
                 numa[1],numa[2],numa[3],numa[4],numa[5],
                 numa[6],numa[7],numa[8],numa[9],numa[10],
                 numa[11],numa[12],numa[13],numa[14],
                 pt,
                 apsis,apsic,aepsc,aepss);
       carg:=0;
       For i:=1 To 14 Do
         carg:=carg+numa[i]*argn[i];
       ca:=Cos(carg);
       sa:=Sin(carg);
       DPsi:=DPsi+apsis*sa+apsic*ca; { in mas }
       DEps:=DEps+aepsc*ca+aepss*sa; { in mas }
     End;
   DPsi:=1.0e-3*DPsi; { in arcsec }
   DEps:=1.0e-3*DEps; { in arcsec }
   Close(fr);
 End;

 END.